import { ReadState, WriteState } from '../state';
import { DeclareChildFeatures, Token, TokenExtension, _childFeatures } from '../token';
import { Error, ErrorsState } from './error';
export declare type ValidationStatus = 'pending' | 'validating' | 'invalid' | 'valid';
declare const _validationStatusToken: unique symbol;
export interface Validated<TState> extends Error<TState> {
    readonly [_validationStatusToken]: Token<ReadState<ValidationStatus> & WriteState<ValidationStatus>>;
    [_childFeatures]?: DeclareChildFeatures<Error<TState>, 'validation', {
        [P in keyof NonNullable<TState>]-?: Validated<NonNullable<TState>[P]>;
    }>;
}
export declare type StateValidator<TState> = (value: TState | undefined) => Promise<ErrorsState>;
export interface ValidationOptions<TState> {
    validator: StateValidator<TState>;
    debounceMs?: number;
}
export declare function addValidation<TState>(options: ValidationOptions<TState>): TokenExtension<ReadState<TState>, Validated<TState>>;
export declare function getTokenValidationStatus(token: Validated<any>): ValidationStatus;
export declare function useTokenValidationStatus(token: Validated<any>): ValidationStatus;
export {};
