import { Dispatch } from 'react';
import { DeclareChildFeatures, NoFeature, Token, TokenExtension, _childFeatures } from '../token';
declare const _getValue: unique symbol;
declare const _subscribe: unique symbol;
declare const _setValue: unique symbol;
export interface State<TReadState, TWriteState = TReadState> extends ReadState<TReadState>, WriteState<TWriteState> {
    [_childFeatures]?: DeclareChildFeatures<ReadState<TReadState> & WriteState<TWriteState>, 'state', {
        [P in keyof NonNullable<TReadState> & keyof NonNullable<TWriteState>]-?: State<NonNullable<TReadState>[P] | (TReadState extends null | undefined ? undefined : never), NonNullable<TWriteState>[P]>;
    }>;
}
export interface ReadState<TState> {
    [_getValue](): TState;
    [_subscribe](listener: TokenValueListener<TState>): Disposer;
    [_childFeatures]?: DeclareChildFeatures<NoFeature, 'readState', {
        [P in keyof NonNullable<TState>]-?: ReadState<NonNullable<TState>[P] | (TState extends null | undefined ? undefined : never)>;
    }>;
}
export interface WriteState<TState> {
    [_setValue](newValue: TState): void;
    [_childFeatures]?: DeclareChildFeatures<NoFeature, 'writeState', {
        [P in keyof NonNullable<TState>]-?: WriteState<NonNullable<TState>[P]>;
    }>;
}
export declare type TokenValueListener<TState> = (newValue: TState) => void;
export declare type Disposer = () => void;
export declare function addState<TState>(initializer: TState | (() => TState)): TokenExtension<NoFeature, State<TState>>;
export declare function getTokenValue<TState>(token: ReadState<TState>): TState;
export declare function setTokenValue<TState>(token: WriteState<TState>, newValue: TState): void;
export declare function subscribeToTokenValue<TState>(token: ReadState<TState>, listener: TokenValueListener<TState>): Disposer;
export declare function useStateToken<T>(initializer: T | (() => T)): Token<State<T>>;
export declare function useTokenValue<TState>(token: ReadState<TState>): TState;
export declare function useTokenSetter<T>(token: WriteState<T>): Dispatch<T>;
export declare function useTokenState<TReadState, TWriteState = TReadState>(token: ReadState<TReadState> & WriteState<TWriteState>): [TReadState, Dispatch<TWriteState>];
export {};
